@@ -2,6 +2,8 @@ package parser
 
 import (
 	"regexp"
+
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/path"
 )
 
 const (
@@ -11,15 +13,19 @@ const (
 	linuxSource = `((?P<source>((` + linuxDir + `)|(` + linuxVolumeName + `))):)?`
 
 	linuxDestination = `(?P<destination>(?:` + linuxDir + `))`
-	linuxMode        = `(:(?P<mode>(?i)ro|rw))?`
+	linuxMode        = `(:(?P<mode>(?i)ro|rw|z))?`
 )
 
 type linuxParser struct {
 	baseParser
 }
 
-func newLinuxParser() Parser {
-	return new(linuxParser)
+func NewLinuxParser() Parser {
+	return &linuxParser{
+		baseParser: baseParser{
+			path: path.NewUnixPath(),
+		},
+	}
 }
 
 func (p *linuxParser) ParseVolume(spec string) (*Volume, error) {
