@@ -30,6 +30,8 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/executors"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers"
 	dns_test "gitlab.com/gitlab-org/gitlab-runner/helpers/dns/test"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker/helperimage"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 	"gitlab.com/gitlab-org/gitlab-runner/session"
 )
 
@@ -1327,7 +1329,7 @@ func TestSetupBuildPod(t *testing.T) {
 				},
 			},
 			Variables: []common.JobVariable{
-				{Key: "FF_K8S_USE_ENTRYPOINT_OVER_COMMAND", Value: "true"},
+				{Key: featureflags.K8sEntrypointOverCommand, Value: "true"},
 			},
 			Options: &kubernetesOptions{
 				Image: common.Image{
@@ -1389,7 +1391,7 @@ func TestSetupBuildPod(t *testing.T) {
 				},
 			},
 			Variables: []common.JobVariable{
-				{Key: "FF_K8S_USE_ENTRYPOINT_OVER_COMMAND", Value: "false"},
+				{Key: featureflags.K8sEntrypointOverCommand, Value: "false"},
 			},
 			Options: &kubernetesOptions{
 				Image: common.Image{
@@ -1459,6 +1461,12 @@ func TestSetupBuildPod(t *testing.T) {
 
 	for testName, test := range tests {
 		t.Run(testName, func(t *testing.T) {
+			helperImageInfo, err := helperimage.Get(common.REVISION, helperimage.Config{
+				OSType:       helperimage.OSTypeLinux,
+				Architecture: "amd64",
+			})
+			require.NoError(t, err)
+
 			vars := test.Variables
 			if vars == nil {
 				vars = []common.JobVariable{}
@@ -1487,13 +1495,14 @@ func TestSetupBuildPod(t *testing.T) {
 						Runner: &test.RunnerConfig,
 					},
 				},
+				helperImageInfo: helperImageInfo,
 			}
 
 			if test.PrepareFn != nil {
 				test.PrepareFn(t, test, &ex)
 			}
 
-			err := ex.prepareOverwrites(make(common.JobVariables, 0))
+			err = ex.prepareOverwrites(make(common.JobVariables, 0))
 			assert.NoError(t, err, "error preparing overwrites")
 
 			err = ex.setupBuildPod()
